(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2004-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: typed_value.mli,v 1.4 2011-02-23 17:43:29 fclement Exp $ *)

(** {3 The values sent and received by the [Io] library} *)

(** This is the definition of the Caml type that specifies which values are
  manipulated in the [Io] library.

 Those values are explicitely tagged in order to accurately encode their
 types. *)

type native_int = Nativeint.t
and int32 = Int32.t
and int64 = Int64.t
;;

type t =
   | Scalar of scalar
   | Vector of vector
   | Matrix of matrix
(* Instead of matrix, should be
   | Multi_d of multi_d *)

and scalar =
  | Sbool of bool
  | Sbool_couple of (bool * bool)
  | Sbool_triple of (bool * bool * bool)
  | Sstring of string
  | Sint of int
  | Sint_couple of (int * int)
  | Sint_triple of (int * int * int)
  | Snative_int of native_int
  | Snative_int_couple of (native_int * native_int)
  | Snative_int_triple of (native_int * native_int * native_int)
  | Sint32 of int32
  | Sint32_couple of (int32 * int32)
  | Sint32_triple of (int32 * int32 * int32)
  | Sint64 of int64
  | Sint64_couple of (int64 * int64)
  | Sint64_triple of (int64 * int64 * int64)
  | Sfloat of float
  | Sfloat_couple of (float * float)
  | Sfloat_triple of (float * float * float)

and vector =
  | Vbool of bool array
  | Vbool_couple of (bool * bool) array
  | Vbool_triple of (bool * bool * bool) array
  | Vstring of string array
  | Vint of int array
  | Vint_couple of (int * int) array
  | Vint_triple of (int * int * int) array
  | Vnative_int of native_int array
  | Vnative_int_couple of (native_int * native_int) array
  | Vnative_int_triple of (native_int * native_int * native_int) array
  | Vint32 of int32 array
  | Vint32_couple of (int32 * int32) array
  | Vint32_triple of (int32 * int32 * int32) array
  | Vint64 of int64 array
  | Vint64_couple of (int64 * int64) array
  | Vint64_triple of (int64 * int64 * int64) array
  | Vfloat of float array
  | Vfloat_couple of (float * float) array
  | Vfloat_triple of (float * float * float) array

and matrix =
  | Mcfloat of float Umatrix.t
  | Mffloat of float Umatrix.t
(** The typed-values that are sent in the communications.
    Those values are explicitely tagged with their types. *)
;;

val type_of_value : t -> Value_type_def.value_type;;
(** The type checker for typed-values. *)

(** {6 Converters from typed values to regular Caml values} *)

exception Wrong_communication of string;;
(** The exception we send when a communication cannot be understood
 (because it is not conform to the DTD for communications). *)

val to_bool : t -> bool;;
val to_bool_couple : t -> bool * bool;;
val to_bool_triple : t -> bool * bool * bool;;
val to_string : t -> string;;
val to_int : t -> int;;
val to_int_couple : t -> int * int;;
val to_int_triple : t -> int * int * int;;
val to_native_int : t -> native_int;;
val to_native_int_couple : t -> native_int * native_int;;
val to_native_int_triple : t -> native_int * native_int * native_int;;
val to_int32 : t -> int32;;
val to_int32_couple : t -> int32 * int32;;
val to_int32_triple : t -> int32 * int32 * int32;;
val to_int64 : t -> int64;;
val to_int64_couple : t -> int64 * int64;;
val to_int64_triple : t -> int64 * int64 * int64;;
val to_float : t -> float;;
val to_float_couple : t -> float * float;;
val to_float_triple : t -> float * float * float;;
val to_bool_vector : t -> bool array;;
val to_bool_couple_vector : t -> (bool * bool) array;;
val to_bool_triple_vector : t -> (bool * bool * bool) array;;
val to_string_vector : t -> string array;;
val to_int_vector : t -> int array;;
val to_int_couple_vector : t -> (int * int) array;;
val to_int_triple_vector : t -> (int * int * int) array;;
val to_native_int_vector : t -> native_int array;;
val to_native_int_couple_vector : t -> (native_int * native_int) array;;
val to_native_int_triple_vector :
  t -> (native_int * native_int * native_int) array;;
val to_int32_vector : t -> int32 array;;
val to_int32_couple_vector : t -> (int32 * int32) array;;
val to_int32_triple_vector : t -> (int32 * int32 * int32) array;;
val to_int64_vector : t -> int64 array;;
val to_int64_couple_vector : t -> (int64 * int64) array;;
val to_int64_triple_vector : t -> (int64 * int64 * int64) array;;
val to_float_vector : t -> float array;;
val to_float_couple_vector : t -> (float * float) array;;
val to_float_triple_vector : t -> (float * float * float) array;;
val to_c_float_matrix : t -> float Umatrix.t;;
val to_fortran_float_matrix : t -> float Umatrix.t;;

(** {6 Converters from regular Caml values to typed values} *)

val of_bool : bool -> t;;
val of_bool_couple : bool * bool -> t;;
val of_bool_triple : bool * bool * bool -> t;;
val of_string : string -> t;;
val of_int : int -> t;;
val of_int_couple : int * int -> t;;
val of_int_triple : int * int * int -> t;;
val of_native_int : native_int -> t;;
val of_native_int_couple : native_int * native_int -> t;;
val of_native_int_triple : native_int * native_int * native_int -> t;;
val of_int32 : int32 -> t;;
val of_int32_couple : int32 * int32 -> t;;
val of_int32_triple : int32 * int32 * int32 -> t;;
val of_int64 : int64 -> t;;
val of_int64_couple : int64 * int64 -> t;;
val of_int64_triple : int64 * int64 * int64 -> t;;
val of_float : float -> t;;
val of_float_couple : float * float -> t;;
val of_float_triple : float * float * float -> t;;
val of_bool_vector : bool array -> t;;
val of_bool_couple_vector : (bool * bool) array -> t;;
val of_bool_triple_vector : (bool * bool * bool) array -> t;;
val of_string_vector : string array -> t;;
val of_int_vector : int array -> t;;
val of_int_couple_vector : (int * int) array -> t;;
val of_int_triple_vector : (int * int * int) array -> t;;
val of_native_int_vector : native_int array -> t;;
val of_native_int_couple_vector : (native_int * native_int) array -> t;;
val of_native_int_triple_vector :
  (native_int * native_int * native_int) array -> t;;
val of_int32_vector : int32 array -> t;;
val of_int32_couple_vector : (int32 * int32) array -> t;;
val of_int32_triple_vector : (int32 * int32 * int32) array -> t;;
val of_int64_vector : int64 array -> t;;
val of_int64_couple_vector : (int64 * int64) array -> t;;
val of_int64_triple_vector : (int64 * int64 * int64) array -> t;;
val of_float_vector : float array -> t;;
val of_float_couple_vector : (float * float) array -> t;;
val of_float_triple_vector : (float * float * float) array -> t;;
val of_c_float_matrix : float Umatrix.t -> t;;
val of_fortran_float_matrix : float Umatrix.t -> t;;
